@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2007-2024 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
+%% Copyright (c) 2007-2025 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
 %%
 
 -module(amqp10_client).
@@ -47,6 +47,7 @@
 
 -type terminus_durability() :: amqp10_client_session:terminus_durability().
 
+-type terminus_address() :: amqp10_client_session:terminus_address().
 -type target_def() :: amqp10_client_session:target_def().
 -type source_def() :: amqp10_client_session:source_def().
 
@@ -64,6 +65,7 @@
               snd_settle_mode/0,
               rcv_settle_mode/0,
               terminus_durability/0,
+              terminus_address/0,
               target_def/0,
               source_def/0,
               attach_role/0,
@@ -168,7 +170,7 @@ attach_sender_link_sync(Session, Name, Target) ->
 %% @doc Synchronously attach a link on 'Session'.
 %% This is a convenience function that awaits attached event
 %% for the link before returning.
--spec attach_sender_link_sync(pid(), binary(), binary(),
+-spec attach_sender_link_sync(pid(), binary(), terminus_address(),
                               snd_settle_mode()) ->
     {ok, link_ref()} | link_timeout.
 attach_sender_link_sync(Session, Name, Target, SettleMode) ->
@@ -177,7 +179,7 @@ attach_sender_link_sync(Session, Name, Target, SettleMode) ->
 %% @doc Synchronously attach a link on 'Session'.
 %% This is a convenience function that awaits attached event
 %% for the link before returning.
--spec attach_sender_link_sync(pid(), binary(), binary(),
+-spec attach_sender_link_sync(pid(), binary(), terminus_address(),
                               snd_settle_mode(), terminus_durability()) ->
     {ok, link_ref()} | link_timeout.
 attach_sender_link_sync(Session, Name, Target, SettleMode, Durability) ->
@@ -195,7 +197,7 @@ attach_sender_link_sync(Session, Name, Target, SettleMode, Durability) ->
 %% This is asynchronous and will notify completion of the attach request to the
 %% caller using an amqp10_event of the following format:
 %% {amqp10_event, {link, LinkRef, attached | {detached, Why}}}
--spec attach_sender_link(pid(), binary(), binary()) -> {ok, link_ref()}.
+-spec attach_sender_link(pid(), binary(), terminus_address()) -> {ok, link_ref()}.
 attach_sender_link(Session, Name, Target) ->
     % mixed should work with any type of msg
     attach_sender_link(Session, Name, Target, mixed).
@@ -204,7 +206,7 @@ attach_sender_link(Session, Name, Target) ->
 %% This is asynchronous and will notify completion of the attach request to the
 %% caller using an amqp10_event of the following format:
 %% {amqp10_event, {link, LinkRef, attached | {detached, Why}}}
--spec attach_sender_link(pid(), binary(), binary(),
+-spec attach_sender_link(pid(), binary(), terminus_address(),
                          snd_settle_mode()) ->
     {ok, link_ref()}.
 attach_sender_link(Session, Name, Target, SettleMode) ->
@@ -214,7 +216,7 @@ attach_sender_link(Session, Name, Target, SettleMode) ->
 %% This is asynchronous and will notify completion of the attach request to the
 %% caller using an amqp10_event of the following format:
 %% {amqp10_event, {link, LinkRef, attached | {detached, Why}}}
--spec attach_sender_link(pid(), binary(), binary(),
+-spec attach_sender_link(pid(), binary(), terminus_address(),
                          snd_settle_mode(), terminus_durability()) ->
     {ok, link_ref()}.
 attach_sender_link(Session, Name, Target, SettleMode, Durability) ->
@@ -335,7 +337,7 @@ flow_link_credit(#link_ref{role = receiver, session = Session,
        RenewWhenBelow =< Credit) ->
     Flow = #'v1_0.flow'{link_credit = {uint, Credit},
                         drain = Drain},
-    ok = amqp10_client_session:flow(Session, Handle, Flow, RenewWhenBelow).
+    ok = amqp10_client_session:flow_link(Session, Handle, Flow, RenewWhenBelow).
 
 %% @doc Stop a receiving link.
 %% See AMQP 1.0 spec §2.6.10.
@@ -344,7 +346,7 @@ stop_receiver_link(#link_ref{role = receiver,
                              link_handle = Handle}) ->
     Flow = #'v1_0.flow'{link_credit = {uint, 0},
                         echo = true},
-    ok = amqp10_client_session:flow(Session, Handle, Flow, never).
+    ok = amqp10_client_session:flow_link(Session, Handle, Flow, never).
 
 %%% messages
 
