@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2018-2020 VMware, Inc. or its affiliates.  All rights reserved.
+%% Copyright (c) 2018-2023 Broadcom. All Rights Reserved. The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
 %%
 -module(aten).
 
@@ -10,23 +10,19 @@
          start/0,
          register/1,
          unregister/1
-         ]).
-
--export_type([
-              ]).
-
+        ]).
 
 start() ->
     application:ensure_all_started(aten).
 
--spec register(node()) -> ok.
+-spec register(node()) -> ok | ignore.
+register(Node) when Node == node() ->
+    ignore;
 register(Node) ->
     aten_detector:register(Node).
 
--spec unregister(node()) -> ok.
+-spec unregister(node()) -> ok | ignore.
+unregister(Node) when Node == node() ->
+    ignore;
 unregister(Node) ->
     aten_detector:unregister(Node).
-
--ifdef(TEST).
--include_lib("eunit/include/eunit.hrl").
--endif.
