@@ -5,15 +5,15 @@
 %% @end
 %%%-------------------------------------------------------------------
 
-%% This module was generated on 2020-04-02T02:00:45+00:00 and should not be modified manually
+%% This module was generated on 2021-12-22T07:01:11+00:00 and should not be modified manually
 
 -module(eetcd_watch_gen).
 
 -export([watch/1]).
 
 %% @doc Stream RPC 
--spec watch(router_pb:'Etcd.WatchRequest'()) ->
-    reference()|{error,eetcd:eetcd_error()}.
+-spec watch(atom()|reference()) ->
+    {ok, GunPid :: pid(), Http2Ref:: reference()}|{error,eetcd:eetcd_error()}.
 watch(Request) ->
     eetcd_stream:new(Request, <<"/etcdserverpb.Watch/Watch">>).
 
