@@ -139,9 +139,7 @@ manager(ChildPid, SheetCache, ViewOpts) ->
                     start(ViewOpts#view_opts{plug = PlugOpts#plug{plugs = NewPlugs}});
                 {error, _} ->
                     manager(ChildPid, SheetCache, ViewOpts)
-            end;
-        _ ->
-            manager(ChildPid, SheetCache, ViewOpts)
+            end
     end.
 
 update_plugins(CurIndex, Lists, UpdateItems) ->
@@ -285,11 +283,11 @@ render_attributes(#{module := Module}, PrevAttrs) ->
                             error -> ?W(Value, Width)
                         end
                     end
-                    || Item <- Label
+                 || Item <- Label
                 ],
                 ?render(L)
             end
-            || Label <- DiffAttrs
+         || Label <- DiffAttrs
         ],
         {Render, length(Render), NewAttrs}
     catch
