@@ -80,6 +80,7 @@ init(UId, Dir, FirstIdx, MaxOpen, AccessPattern, SegRefs,
                           _ = ra_log_segment:close(Seg),
                           decr_counter(Cfg, ?C_RA_LOG_OPEN_SEGMENTS, 1)
                   end,
+    reset_counter(Cfg, ?C_RA_LOG_OPEN_SEGMENTS),
     #?STATE{cfg = Cfg,
             open_segments = ra_flru:new(MaxOpen, FlruHandler),
             first_index = FirstIdx,
@@ -480,6 +481,12 @@ compact_seg_refs(NewSegRefs,
             NewSegRefs ++ PreviousSegRefs
     end.
 
+reset_counter(#cfg{counter = Cnt}, Ix)
+  when Cnt =/= undefined ->
+    counters:put(Cnt, Ix, 0);
+reset_counter(#cfg{counter = undefined}, _) ->
+    ok.
+
 incr_counter(#cfg{counter = Cnt}, Ix, N) when Cnt =/= undefined ->
     counters:add(Cnt, Ix, N);
 incr_counter(#cfg{counter = undefined}, _, _) ->
