@@ -33,7 +33,7 @@
 %% gen_statem callbacks
 -export([
          init/1,
-         format_status/2,
+         format_status/1,
          handle_event/4,
          terminate/3,
          code_change/4,
@@ -72,6 +72,19 @@
 -define(HANDLE_EFFECTS(Effects, EvtType, State0),
         handle_effects(?FUNCTION_NAME, Effects, EvtType, State0)).
 
+-define(ASYNC_DIST(Node, Send),
+        case Node == node() of
+            true ->
+                Send,
+                ok;
+            false ->
+                %% use async_dist for remote sends
+                process_flag(async_dist, true),
+                Send,
+                process_flag(async_dist, false),
+                ok
+        end).
+
 -type query_fun() :: ra:query_fun().
 -type query_options() :: #{condition => ra:query_condition()}.
 
@@ -874,10 +887,15 @@ receive_snapshot(EventType, Msg, State0) ->
         {receive_snapshot, State1, Effects} ->
             {#state{conf = Conf} = State, Actions} =
                 ?HANDLE_EFFECTS(Effects, EventType, State1),
-            #conf{receive_snapshot_timeout = ReceiveSnapshotTimeout} = Conf,
-            {keep_state, State,
-             [{state_timeout, ReceiveSnapshotTimeout,
-               receive_snapshot_timeout} | Actions]};
+            TimeoutActions = case Msg of
+                                 #install_snapshot_rpc{} ->
+                                     %% Reset timeout only on receive snapshot progress.
+                                     [{state_timeout, Conf#conf.receive_snapshot_timeout,
+                                                      receive_snapshot_timeout}];
+                                 _ ->
+                                     []
+                             end,
+            {keep_state, State, TimeoutActions ++ Actions};
         {follower, State1, Effects} ->
             {State2, Actions} = ?HANDLE_EFFECTS(Effects, EventType, State1),
             State = follower_leader_change(State0, State2),
@@ -1087,26 +1105,26 @@ terminate(Reason, StateName, State) ->
 code_change(_OldVsn, StateName, State, _Extra) ->
     {ok, StateName, State}.
 
-format_status(Opt, [_PDict, StateName,
-                    #state{server_state = NS,
-                           leader_last_seen = LastSeen,
-                           pending_commands = Pending,
-                           low_priority_commands = Delayed,
-                           pending_notifys = PendingNots,
-                           election_timeout_set = ElectionSet
-                          }]) ->
+format_status(#{state := StateName,
+                data := #state{server_state = NS,
+                               leader_last_seen = LastSeen,
+                               pending_commands = Pending,
+                               low_priority_commands = Delayed,
+                               pending_notifys = PendingNots,
+                               election_timeout_set = ElectionSet
+                              }} = FormatStatus) ->
     NumPendingNots = maps:fold(fun (_, Corrs, Acc) -> Acc + length(Corrs) end,
                                0, PendingNots),
-    [{id, ra_server:id(NS)},
-     {opt, Opt},
-     {raft_state, StateName},
-     {leader_last_seen, LastSeen},
-     {num_pending_commands, length(Pending)},
-     {num_low_priority_commands, ra_ets_queue:len(Delayed)},
-     {num_pending_applied_notifications, NumPendingNots},
-     {election_timeout_set, ElectionSet},
-     {ra_server_state, ra_server:overview(NS)}
-    ].
+
+    FormatStatus#{data => [{id, ra_server:id(NS)},
+                           {raft_state, StateName},
+                           {leader_last_seen, LastSeen},
+                           {num_pending_commands, length(Pending)},
+                           {num_low_priority_commands, ra_ets_queue:len(Delayed)},
+                           {num_pending_applied_notifications, NumPendingNots},
+                           {election_timeout_set, ElectionSet},
+                           {ra_server_state, ra_server:overview(NS)}
+                          ]}.
 
 %%%===================================================================
 %%% Internal functions
@@ -1120,17 +1138,20 @@ handle_enter(RaftState, OldRaftState,
     {ServerState, Effects} = ra_server:handle_state_enter(RaftState,
                                                           OldRaftState,
                                                           ServerState0),
+    LastApplied = do_state_query(last_applied, State),
     case RaftState == leader orelse OldRaftState == leader of
         true ->
             %% ensure transitions from and to leader are logged at a higher
             %% level
-            ?NOTICE("~ts: ~s -> ~s in term: ~b machine version: ~b",
+            ?NOTICE("~ts: ~s -> ~s in term: ~b machine version: ~b, last applied ~b",
                     [log_id(State), OldRaftState, RaftState,
-                     current_term(State), machine_version(State)]);
+                     current_term(State), machine_version(State),
+                     LastApplied]);
         false ->
-            ?DEBUG("~ts: ~s -> ~s in term: ~b machine version: ~b",
+            ?DEBUG("~ts: ~s -> ~s in term: ~b machine version: ~b, last applied ~b",
                    [log_id(State), OldRaftState, RaftState,
-                    current_term(State), machine_version(State)])
+                    current_term(State), machine_version(State),
+                    LastApplied])
     end,
     handle_effects(RaftState, Effects, cast,
                    State#state{server_state = ServerState}).
@@ -1340,8 +1361,8 @@ handle_effect(_, {next_event, _, _} = Next, _, State, Actions) ->
     {State, [Next | Actions]};
 handle_effect(_, {send_msg, To, Msg}, _, State, Actions) ->
     %% default is to send without any wrapping
-    %% TODO: handle send failure? how?
-    _ = send(To, Msg, State#state.conf),
+    ToNode = get_node(To),
+    ?ASYNC_DIST(ToNode, _ = send(To, Msg, State#state.conf)),
     {State, Actions};
 handle_effect(RaftState, {send_msg, To, _Msg, Options} = Eff,
               _, State, Actions) ->
@@ -1350,13 +1371,13 @@ handle_effect(RaftState, {send_msg, To, _Msg, Options} = Eff,
         true ->
             case can_execute_locally(RaftState, ToNode, State) of
                 true ->
-                    send_msg(Eff, State);
+                    ?ASYNC_DIST(ToNode, send_msg(Eff, State));
                 false ->
                     ok
             end;
         false when RaftState == leader ->
             %% the effect got here so we can execute
-            send_msg(Eff, State);
+            ?ASYNC_DIST(ToNode, send_msg(Eff, State));
         false ->
             ok
     end,
@@ -1930,10 +1951,10 @@ handle_tick_metrics(State) ->
 
 can_execute_locally(RaftState, TargetNode,
                     #state{server_state = ServerState} = State) ->
-    Membership = ra_server:get_membership(ServerState),
     case RaftState of
-        follower when Membership == voter ->
-            TargetNode == node();
+        follower  ->
+            TargetNode == node() andalso
+            voter == ra_server:get_membership(ServerState);
         leader when TargetNode =/= node() ->
             %% We need to evaluate whether to send the message.
             %% Only send if there isn't a local node for the target pid.
