@@ -8,7 +8,7 @@
 
 %% mc
 -export([
-         init/2,
+         init/1,
          size/1,
          x_header/2,
          routing_headers/2,
@@ -24,8 +24,9 @@
 -export([
          message/3,
          message/4,
-         message/5,
-         from_basic_message/1
+         from_basic_message/1,
+         to_091/2,
+         from_091/2
         ]).
 
 -import(rabbit_misc,
@@ -38,6 +39,7 @@
 -define(AMQP10_PROPERTIES_HEADER, <<"x-amqp-1.0-properties">>).
 -define(AMQP10_APP_PROPERTIES_HEADER, <<"x-amqp-1.0-app-properties">>).
 -define(AMQP10_MESSAGE_ANNOTATIONS_HEADER, <<"x-amqp-1.0-message-annotations">>).
+-define(AMQP10_FOOTER, <<"x-amqp-1.0-footer">>).
 -define(PROTOMOD, rabbit_framing_amqp_0_9_1).
 -define(CLASS_ID, 60).
 -define(LONGSTR_UTF8_LIMIT, 4096).
@@ -49,56 +51,61 @@
              ]).
 
 %% mc implementation
-init(#content{} = Content0, Env) ->
+init(#content{} = Content0) ->
     Content1 = rabbit_binary_parser:ensure_content_decoded(Content0),
     %% project essential properties into annotations
-    Anns = essential_properties(Content1, Env),
+    Anns = essential_properties(Content1),
     Content = strip_header(Content1, ?DELETED_HEADER),
     {Content, Anns}.
 
-convert_from(mc_amqp, Sections, _Env) ->
-    {H, MAnn, Prop, AProp, BodyRev} =
-        lists:foldl(
-          fun
-              (#'v1_0.header'{} = S, Acc) ->
-                  setelement(1, Acc, S);
-              (#'v1_0.message_annotations'{} = S, Acc) ->
-                  setelement(2, Acc, S);
-              (#'v1_0.properties'{} = S, Acc) ->
-                  setelement(3, Acc, S);
-              (#'v1_0.application_properties'{} = S, Acc) ->
-                  setelement(4, Acc, S);
-              (#'v1_0.delivery_annotations'{}, Acc) ->
-                  %% delivery annotations not currently used
-                  Acc;
-              (#'v1_0.footer'{}, Acc) ->
-                  %% footer not currently used
-                  Acc;
-              (undefined, Acc) ->
-                  Acc;
-              (BodySection, Acc) ->
-                  Body = element(5, Acc),
-                  setelement(5, Acc, [BodySection | Body])
-          end, {undefined, undefined, undefined, undefined, []},
-          Sections),
+convert_from(mc_amqp, Sections, Env) ->
+    {H, MAnn, Prop, AProp, BodyRev, Footer} =
+    lists:foldl(
+      fun(#'v1_0.header'{} = S, Acc) ->
+              setelement(1, Acc, S);
+         (_Ignore = #'v1_0.delivery_annotations'{}, Acc) ->
+              Acc;
+         (#'v1_0.message_annotations'{} = S, Acc) ->
+              setelement(2, Acc, S);
+         (#'v1_0.properties'{} = S, Acc) ->
+              setelement(3, Acc, S);
+         (#'v1_0.application_properties'{} = S, Acc) ->
+              setelement(4, Acc, S);
+         (BodySect, Acc)
+           when is_record(BodySect, 'v1_0.data') orelse
+                is_record(BodySect, 'v1_0.amqp_sequence') orelse
+                is_record(BodySect, 'v1_0.amqp_value') ->
+              Body = element(5, Acc),
+              setelement(5, Acc, [BodySect | Body]);
+         (Body = {amqp_encoded_body_and_footer, _}, Acc) ->
+              %% assertions
+              [] = element(5, Acc),
+              setelement(5, Acc, Body);
+         (#'v1_0.footer'{} = S, Acc) ->
+              setelement(6, Acc, S)
+      end,
+      {undefined, undefined, undefined, undefined, [], undefined},
+      Sections),
 
-    {PayloadRev, Type0} =
-        case BodyRev of
-            [#'v1_0.data'{content = Bin}] when is_binary(Bin) ->
-                {[Bin], undefined};
-            [#'v1_0.data'{content = Bin}] when is_list(Bin) ->
-                {lists:reverse(Bin), undefined};
-            _ ->
-                %% anything else needs to be encoded
-                %% TODO: This is potentially inefficient, but #content.payload_fragments_rev expects
-                %% currently a flat list of binaries. Can we make rabbit_writer work
-                %% with an iolist instead?
-                BinsRev = [begin
-                               IoList = amqp10_framing:encode_bin(X),
-                               erlang:iolist_to_binary(IoList)
-                           end || X <- BodyRev],
-                {BinsRev, ?AMQP10_TYPE}
-        end,
+    {PFR, Type0} = case BodyRev of
+                       [#'v1_0.data'{} | _] ->
+                           %% We assert that the body consists of one or more data sections.
+                           %% If there are multiple data sections, we concatenate the binary data.
+                           PFR0 = lists:map(
+                                    fun(#'v1_0.data'{content = Content}) ->
+                                            %% In practice, when converting from mc_amqp
+                                            %% to mc_amqpl, Content will be a single binary,
+                                            %% in which case iolist_to_binary/1 is cheap.
+                                            iolist_to_binary(Content)
+                                    end, BodyRev),
+                           {PFR0, undefined};
+                       {amqp_encoded_body_and_footer, BodyAndFooterBin} ->
+                           {[BodyAndFooterBin], ?AMQP10_TYPE};
+                       _ ->
+                           %% Anything else needs to be AMQP encoded.
+                           PFR0 = lists:map(fun amqp_encoded_binary/1, BodyRev),
+                           {PFR0, ?AMQP10_TYPE}
+                   end,
     #'v1_0.properties'{message_id = MsgId,
                        user_id = UserId0,
                        reply_to = ReplyTo0,
@@ -166,7 +173,42 @@ convert_from(mc_amqp, Sections, _Env) ->
                                        false
                                end, MA),
     {Headers1, MsgId091} = message_id(MsgId, <<"x-message-id">>, Headers0),
-    {Headers, CorrId091} = message_id(CorrId, <<"x-correlation-id">>, Headers1),
+    {Headers2, CorrId091} = message_id(CorrId, <<"x-correlation-id">>, Headers1),
+
+    Headers = case Env of
+                  #{'rabbitmq_4.0.0' := false} ->
+                      Headers3 = case AProp of
+                                     undefined ->
+                                         Headers2;
+                                     #'v1_0.application_properties'{} ->
+                                         APropBin = amqp_encoded_binary(AProp),
+                                         [{?AMQP10_APP_PROPERTIES_HEADER, longstr, APropBin} | Headers2]
+                                 end,
+                      Headers4 = case Prop of
+                                     undefined ->
+                                         Headers3;
+                                     #'v1_0.properties'{} ->
+                                         PropBin = amqp_encoded_binary(Prop),
+                                         [{?AMQP10_PROPERTIES_HEADER, longstr, PropBin} | Headers3]
+                                 end,
+                      Headers5 = case MAnn of
+                                     undefined ->
+                                         Headers4;
+                                     #'v1_0.message_annotations'{} ->
+                                         MAnnBin = amqp_encoded_binary(MAnn),
+                                         [{?AMQP10_MESSAGE_ANNOTATIONS_HEADER, longstr, MAnnBin} | Headers4]
+                                 end,
+                      Headers6 = case Footer of
+                                     undefined ->
+                                         Headers5;
+                                     #'v1_0.footer'{} ->
+                                         FootBin = amqp_encoded_binary(Footer),
+                                         [{?AMQP10_FOOTER, longstr, FootBin} | Headers5]
+                                 end,
+                      Headers6;
+                  _ ->
+                      Headers2
+              end,
 
     UserId1 = unwrap(UserId0),
     %% user-id is a binary type so we need to validate
@@ -179,7 +221,7 @@ convert_from(mc_amqp, Sections, _Env) ->
                      undefined
              end,
 
-    BP = #'P_basic'{message_id =  MsgId091,
+    BP = #'P_basic'{message_id = MsgId091,
                     delivery_mode = DelMode,
                     expiration = Expiration,
                     user_id = UserId,
@@ -199,7 +241,7 @@ convert_from(mc_amqp, Sections, _Env) ->
     #content{class_id = ?CLASS_ID,
              properties = BP,
              properties_bin = none,
-             payload_fragments_rev = PayloadRev};
+             payload_fragments_rev = PFR};
 convert_from(_SourceProto, _, _) ->
     not_implemented.
 
@@ -275,7 +317,7 @@ prepare(store, Content) ->
 
 convert_to(?MODULE, Content, _Env) ->
     Content;
-convert_to(mc_amqp, #content{payload_fragments_rev = Payload} = Content, Env) ->
+convert_to(mc_amqp, #content{payload_fragments_rev = PFR} = Content, Env) ->
     #content{properties = Props} = prepare(read, Content),
     #'P_basic'{message_id = MsgId0,
                expiration = Expiration,
@@ -380,21 +422,28 @@ convert_to(mc_amqp, #content{payload_fragments_rev = Payload} = Content, Env) ->
              Section ->
                  Section
          end,
-
     BodySections = case Type of
                        ?AMQP10_TYPE ->
                            amqp10_framing:decode_bin(
-                             iolist_to_binary(lists:reverse(Payload)));
+                             iolist_to_binary(lists:reverse(PFR)));
                        _ ->
-                           [#'v1_0.data'{content = lists:reverse(Payload)}]
+                           [#'v1_0.data'{content = lists:reverse(PFR)}]
                    end,
+    Tail = case amqp10_section_header(?AMQP10_FOOTER, Headers) of
+               undefined ->
+                   BodySections;
+               #'v1_0.footer'{} = Footer ->
+                   BodySections ++ [Footer]
+           end,
 
-    Sections = [H, MA, P, AP | BodySections],
+    Sections = [H, MA, P, AP | Tail],
     mc_amqp:convert_from(mc_amqp, Sections, Env);
 convert_to(_TargetProto, _Content, _Env) ->
     not_implemented.
 
-protocol_state(#content{properties = #'P_basic'{headers = H00} = B0} = C,
+protocol_state(#content{properties = #'P_basic'{headers = H00,
+                                                priority = Priority0,
+                                                delivery_mode = DeliveryMode0} = B0} = C,
                Anns) ->
     %% Add any x- annotations as headers
     H0 = case H00 of
@@ -441,16 +490,44 @@ protocol_state(#content{properties = #'P_basic'{headers = H00} = B0} = C,
                          %% publishes
                          undefined;
                      #{ttl := Ttl} ->
-                         %% not sure this will ever happen
-                         %% as we only ever unset the expiry
                          integer_to_binary(Ttl);
                      _ ->
                          B0#'P_basic'.expiration
                  end,
-
-    B = B0#'P_basic'{timestamp = Timestamp,
+    Priority = case Priority0 of
+                   undefined ->
+                       case Anns of
+                           #{?ANN_PRIORITY := P} ->
+                               %% This branch is hit when a message with priority was originally
+                               %% published with AMQP to a classic or quorum queue because the
+                               %% AMQP header isn't stored on disk.
+                               P;
+                           _ ->
+                               undefined
+                       end;
+                   _ ->
+                       Priority0
+               end,
+    DelMode = case DeliveryMode0 of
+                  undefined ->
+                      case Anns of
+                          #{?ANN_DURABLE := false} ->
+                              %% Leave it undefined which is equivalent to 1.
+                              undefined;
+                          _ ->
+                              %% This branch is hit when a durable message was originally published
+                              %% with AMQP to a classic or quorum queue because the AMQP header isn't
+                              %% stored on disk.
+                              2
+                      end;
+                  _ ->
+                      DeliveryMode0
+              end,
+    B = B0#'P_basic'{headers = Headers,
+                     delivery_mode = DelMode,
+                     priority = Priority,
                      expiration = Expiration,
-                     headers = Headers},
+                     timestamp = Timestamp},
 
     C#content{properties = B,
               properties_bin = none};
@@ -465,16 +542,13 @@ protocol_state(Content0, Anns) ->
 message(ExchangeName, RoutingKey, Content) ->
     message(ExchangeName, RoutingKey, Content, #{}).
 
+%% helper for creating message container from messages received from AMQP legacy
 -spec message(rabbit_types:exchange_name(), rabbit_types:routing_key(), #content{}, map()) ->
     {ok, mc:state()} | {error, Reason :: any()}.
-message(XName, RoutingKey, Content, Anns) ->
-    message(XName, RoutingKey, Content, Anns,
-            rabbit_feature_flags:is_enabled(message_containers)).
-
-%% helper for creating message container from messages received from
-%% AMQP legacy
-message(#resource{name = ExchangeNameBin}, RoutingKey,
-        #content{properties = Props} = Content, Anns, true)
+message(#resource{name = ExchangeNameBin},
+        RoutingKey,
+        #content{properties = Props} = Content,
+        Anns)
   when is_binary(RoutingKey) andalso
        is_map(Anns) ->
     case rabbit_basic:header_routes(Props#'P_basic'.headers) of
@@ -484,21 +558,7 @@ message(#resource{name = ExchangeNameBin}, RoutingKey,
             {ok, mc:init(?MODULE,
                          Content,
                          Anns#{?ANN_ROUTING_KEYS => [RoutingKey | HeaderRoutes],
-                               ?ANN_EXCHANGE => ExchangeNameBin},
-                         ?MC_ENV)}
-    end;
-message(#resource{} = XName, RoutingKey,
-        #content{} = Content, Anns, false) ->
-    case rabbit_basic:message(XName, RoutingKey, Content) of
-        {ok, Msg} ->
-            case Anns of
-                #{id := Id} ->
-                    {ok, Msg#basic_message{id = Id}};
-                _ ->
-                    {ok, Msg}
-            end;
-        {error, _} = Error ->
-            Error
+                               ?ANN_EXCHANGE => ExchangeNameBin})}
     end.
 
 from_basic_message(#basic_message{content = Content,
@@ -511,11 +571,80 @@ from_basic_message(#basic_message{content = Content,
                _ ->
                    #{id => Id}
            end,
-    {ok, Msg} = message(Ex, RKey, prepare(read, Content), Anns, true),
+    {ok, Msg} = message(Ex, RKey, prepare(read, Content), Anns),
     Msg.
 
 %% Internal
 
+deaths_to_headers(Deaths, Headers0) ->
+    Infos = case Deaths of
+                #deaths{records = Records} ->
+                    %% sort records by the last timestamp
+                    List = lists:sort(
+                             fun({_, #death{anns = #{last_time := L1}}},
+                                 {_, #death{anns = #{last_time := L2}}}) ->
+                                     L1 =< L2
+                             end, maps:to_list(Records)),
+                    lists:foldl(fun(Record, Acc) ->
+                                        Table = death_table(Record),
+                                        [Table | Acc]
+                                end, [], List);
+                _ ->
+                    lists:map(fun death_table/1, Deaths)
+            end,
+    rabbit_misc:set_table_value(Headers0, <<"x-death">>, array, Infos).
+
+convert_from_amqp_deaths({array, map, Maps}) ->
+    L = lists:map(
+          fun({map, KvList}) ->
+                  {Ttl, KvList1} = case KvList of
+                                       [{{symbol, <<"ttl">>}, {uint, Ttl0}} | Tail] ->
+                                           {Ttl0, Tail};
+                                       _ ->
+                                           {undefined, KvList}
+                                   end,
+                  [
+                   {{symbol, <<"queue">>}, {utf8, Queue}},
+                   {{symbol, <<"reason">>}, {symbol, Reason}},
+                   {{symbol, <<"count">>}, {ulong, Count}},
+                   {{symbol, <<"first-time">>}, {timestamp, FirstTime}},
+                   {{symbol, <<"last-time">>}, {timestamp, _LastTime}},
+                   {{symbol, <<"exchange">>}, {utf8, Exchange}},
+                   {{symbol, <<"routing-keys">>}, {array, utf8, RKeys0}}
+                  ] = KvList1,
+                  RKeys = [Key || {utf8, Key} <- RKeys0],
+                  death_table(Queue, Reason, Exchange, RKeys, Count, FirstTime, Ttl)
+          end, Maps),
+    {true, {<<"x-death">>, array, L}};
+convert_from_amqp_deaths(_IgnoreUnknownValue) ->
+    false.
+
+death_table({{QName, Reason},
+             #death{exchange = Exchange,
+                    routing_keys = RoutingKeys,
+                    count = Count,
+                    anns = DeathAnns = #{first_time := FirstTime}}}) ->
+    death_table(QName, Reason, Exchange, RoutingKeys, Count, FirstTime,
+                maps:get(ttl, DeathAnns, undefined)).
+
+death_table(QName, Reason, Exchange, RoutingKeys, Count, FirstTime, Ttl) ->
+    L0 = [
+          {<<"count">>, long, Count},
+          {<<"reason">>, longstr, rabbit_data_coercion:to_binary(Reason)},
+          {<<"queue">>, longstr, QName},
+          {<<"time">>, timestamp, FirstTime div 1000},
+          {<<"exchange">>, longstr, Exchange},
+          {<<"routing-keys">>, array, [{longstr, Key} || Key <- RoutingKeys]}
+         ],
+    L = case Ttl of
+            undefined ->
+                L0;
+            _ ->
+                Expiration = integer_to_binary(Ttl),
+                [{<<"original-expiration">>, longstr, Expiration} | L0]
+        end,
+    {table, L}.
+
 strip_header(#content{properties = #'P_basic'{headers = undefined}}
              = DecodedContent, _Key) ->
     DecodedContent;
@@ -625,9 +754,14 @@ to_091(Key, false) -> {Key, bool, false};
 to_091(Key, undefined) -> {Key, void, undefined};
 to_091(Key, null) -> {Key, void, undefined};
 to_091(Key, {list, L}) ->
-    {Key, array, [to_091(V) || V <- L]};
+    to_091_array(Key, L);
 to_091(Key, {map, M}) ->
-    {Key, table, [to_091(unwrap(K), V) || {K, V} <- M]}.
+    {Key, table, [to_091(unwrap(K), V) || {K, V} <- M]};
+to_091(Key, {array, _T, L}) ->
+    to_091_array(Key, L).
+
+to_091_array(Key, L) ->
+    {Key, array, [to_091(V) || V <- L]}.
 
 to_091({utf8, V}) -> {longstr, V};
 to_091({symbol, V}) -> {longstr, V};
@@ -668,11 +802,11 @@ message_id({utf8, S}, HKey, H0) ->
 message_id(undefined, _HKey, H) ->
     {H, undefined}.
 
-essential_properties(#content{properties = Props}, Env) ->
+essential_properties(#content{} = C) ->
     #'P_basic'{delivery_mode = Mode,
                priority = Priority,
                timestamp = TimestampRaw,
-               headers = Headers} = Props,
+               headers = Headers} = Props = C#content.properties,
     {ok, MsgTTL} = rabbit_basic:parse_expiration(Props),
     Timestamp = case TimestampRaw of
                     undefined ->
@@ -688,8 +822,6 @@ essential_properties(#content{properties = Props}, Env) ->
                   _ ->
                       undefined
               end,
-    Deaths = headers_to_deaths(Headers, Env),
-
     maps_put_truthy(
       ?ANN_PRIORITY, Priority,
       maps_put_truthy(
@@ -700,127 +832,7 @@ essential_properties(#content{properties = Props}, Env) ->
             ?ANN_DURABLE, Durable,
             maps_put_truthy(
               bcc, BccKeys,
-              maps_put_truthy(
-                deaths, Deaths,
-                #{})))))).
-
-headers_to_deaths(_, #{?FF_MC_DEATHS_V2 := false}) ->
-    undefined;
-headers_to_deaths(undefined, _) ->
-    undefined;
-headers_to_deaths(Headers, _) ->
-    case lists:keymember(<<"x-death">>, 1, Headers) of
-        true ->
-            case rabbit_misc:amqp_table(Headers) of
-                #{<<"x-death">> := XDeathList}
-                  when is_list(XDeathList) ->
-                    recover_deaths(XDeathList, []);
-                _ ->
-                    undefined
-            end;
-        false ->
-            undefined
-    end.
-
-recover_deaths([], Acc) ->
-    lists:reverse(Acc);
-recover_deaths([Map = #{<<"exchange">> := Exchange,
-                        <<"queue">> := Queue,
-                        <<"routing-keys">> := RKeys,
-                        <<"reason">> := ReasonBin,
-                        <<"count">> := Count,
-                        <<"time">> := TsSeconds} | Rem], Acc0)
-  when is_integer(TsSeconds) ->
-    Reason = binary_to_existing_atom(ReasonBin),
-    TsMillis = TsSeconds * 1000,
-    DeathAnns0 = #{first_time => TsMillis,
-                   %% Given that this timestamp is absent in the AMQP 0.9.1
-                   %% x-death header, the last_time we set here is incorrect
-                   %% if the message was dead lettered more than one time.
-                   last_time => TsMillis},
-    DeathAnns = case Map of
-                    #{<<"original-expiration">> := Exp} ->
-                        DeathAnns0#{ttl => binary_to_integer(Exp)};
-                    _ ->
-                        DeathAnns0
-                end,
-    Acc = [{{Queue, Reason},
-            #death{anns = DeathAnns,
-                   exchange = Exchange,
-                   count = Count,
-                   routing_keys = RKeys}} | Acc0],
-    recover_deaths(Rem, Acc);
-recover_deaths([_IgnoreInvalid | Rem], Acc) ->
-    recover_deaths(Rem, Acc).
-
-deaths_to_headers(Deaths, Headers0) ->
-    Infos = case Deaths of
-                #deaths{records = Records} ->
-                    %% sort records by the last timestamp
-                    List = lists:sort(
-                             fun({_, #death{anns = #{last_time := L1}}},
-                                 {_, #death{anns = #{last_time := L2}}}) ->
-                                     L1 =< L2
-                             end, maps:to_list(Records)),
-                    lists:foldl(fun(Record, Acc) ->
-                                        Table = death_table(Record),
-                                        [Table | Acc]
-                                end, [], List);
-                _ ->
-                    lists:map(fun death_table/1, Deaths)
-            end,
-    rabbit_misc:set_table_value(Headers0, <<"x-death">>, array, Infos).
-
-convert_from_amqp_deaths({array, map, Maps}) ->
-    L = lists:map(
-          fun({map, KvList}) ->
-                  {Ttl, KvList1} = case KvList of
-                                       [{{symbol, <<"ttl">>}, {uint, Ttl0}} | Tail] ->
-                                           {Ttl0, Tail};
-                                       _ ->
-                                           {undefined, KvList}
-                                   end,
-                  [
-                   {{symbol, <<"queue">>}, {utf8, Queue}},
-                   {{symbol, <<"reason">>}, {symbol, Reason}},
-                   {{symbol, <<"count">>}, {ulong, Count}},
-                   {{symbol, <<"first-time">>}, {timestamp, FirstTime}},
-                   {{symbol, <<"last-time">>}, {timestamp, _LastTime}},
-                   {{symbol, <<"exchange">>}, {utf8, Exchange}},
-                   {{symbol, <<"routing-keys">>}, {array, utf8, RKeys0}}
-                  ] = KvList1,
-                  RKeys = [Key || {utf8, Key} <- RKeys0],
-                  death_table(Queue, Reason, Exchange, RKeys, Count, FirstTime, Ttl)
-          end, Maps),
-    {true, {<<"x-death">>, array, L}};
-convert_from_amqp_deaths(_IgnoreUnknownValue) ->
-    false.
-
-death_table({{QName, Reason},
-             #death{exchange = Exchange,
-                    routing_keys = RoutingKeys,
-                    count = Count,
-                    anns = DeathAnns = #{first_time := FirstTime}}}) ->
-    death_table(QName, Reason, Exchange, RoutingKeys, Count, FirstTime,
-                maps:get(ttl, DeathAnns, undefined)).
-
-death_table(QName, Reason, Exchange, RoutingKeys, Count, FirstTime, Ttl) ->
-    L0 = [
-          {<<"count">>, long, Count},
-          {<<"reason">>, longstr, rabbit_data_coercion:to_binary(Reason)},
-          {<<"queue">>, longstr, QName},
-          {<<"time">>, timestamp, FirstTime div 1000},
-          {<<"exchange">>, longstr, Exchange},
-          {<<"routing-keys">>, array, [{longstr, Key} || Key <- RoutingKeys]}
-         ],
-    L = case Ttl of
-            undefined ->
-                L0;
-            _ ->
-                Expiration = integer_to_binary(Ttl),
-                [{<<"original-expiration">>, longstr, Expiration} | L0]
-        end,
-    {table, L}.
+              #{}))))).
 
 %% headers that are added as annotations during conversions
 is_internal_header(<<"x-basic-", _/binary>>) ->
@@ -842,3 +854,6 @@ amqp10_section_header(Header, Headers) ->
         _ ->
             undefined
     end.
+
+amqp_encoded_binary(Section) ->
+    iolist_to_binary(amqp10_framing:encode_bin(Section)).
