@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2007-2024 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
+%% Copyright (c) 2007-2025 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
 %%
 
 -module(rabbit_peer_discovery).
@@ -22,6 +22,7 @@
          maybe_unregister/0,
          discover_cluster_nodes/0]).
 -export([backend/0,
+         should_perform_registration/0,
          node_type/0,
          normalize/1,
          append_node_prefix/1,
@@ -46,6 +47,9 @@
 %% a new cluster as a virgin node
 -define(DEFAULT_NODE_TYPE, disc).
 
+%% Register node by default (with the backends that support registration)
+-define(PERFORM_REGISTRATION_BY_DEFAULT, true).
+
 %% default node prefix to attach to discovered hostnames
 -define(DEFAULT_PREFIX, "rabbit").
 
@@ -82,6 +86,16 @@ node_type() ->
             ?DEFAULT_NODE_TYPE
     end.
 
+-spec should_perform_registration() -> true | false.
+
+should_perform_registration() ->
+    case application:get_env(rabbit, cluster_formation) of
+        {ok, Proplist} ->
+            proplists:get_value(perform_registration, Proplist, ?PERFORM_REGISTRATION_BY_DEFAULT);
+        undefined ->
+            ?PERFORM_REGISTRATION_BY_DEFAULT
+    end.
+
 -spec lock_acquisition_failure_mode() -> ignore | fail.
 
 lock_acquisition_failure_mode() ->
@@ -613,8 +627,7 @@ query_node_props2([{Node, Members} | Rest], NodesAndProps, FromNode) ->
                            ["Peer discovery: temporary hidden node '~ts' "
                             "queries properties from node '~ts'",
                             [node(), Node]], FromNode),
-                         StartTime = get_node_start_time(
-                                       Node, microsecond, FromNode),
+                         StartTime = get_node_start_time(Node, FromNode),
                          IsReady = is_node_db_ready(Node, FromNode),
                          NodeAndProps = {Node, Members, StartTime, IsReady},
                          NodesAndProps1 = [NodeAndProps | NodesAndProps],
@@ -642,9 +655,8 @@ query_node_props2([], NodesAndProps, _FromNode) ->
     ?assert(length(NodesAndProps1) =< length(nodes(hidden))),
     NodesAndProps1.
 
--spec get_node_start_time(Node, Unit, FromNode) -> StartTime when
+-spec get_node_start_time(Node, FromNode) -> StartTime when
       Node :: node(),
-      Unit :: erlang:time_unit(),
       FromNode :: node(),
       StartTime :: non_neg_integer().
 %% @doc Returns the start time of the given `Node' in `Unit'.
@@ -665,15 +677,21 @@ query_node_props2([], NodesAndProps, _FromNode) ->
 %%
 %% @private
 
-get_node_start_time(Node, Unit, FromNode) ->
-    NativeStartTime = erpc_call(
-                        Node, erlang, system_info, [start_time], FromNode),
-    TimeOffset = erpc_call(Node, erlang, time_offset, [], FromNode),
-    SystemStartTime = NativeStartTime + TimeOffset,
-    StartTime = erpc_call(
-                  Node, erlang, convert_time_unit,
-                  [SystemStartTime, native, Unit], FromNode),
-    StartTime.
+get_node_start_time(Node, FromNode) ->
+    try
+        erpc_call(Node,rabbit_boot_state, get_start_time, [], FromNode)
+    catch
+        error:{exception, _, _} ->
+            NativeStartTime = erpc_call(
+                                Node, erlang, system_info, [start_time],
+                                FromNode),
+            TimeOffset = erpc_call(Node, erlang, time_offset, [], FromNode),
+            SystemStartTime = NativeStartTime + TimeOffset,
+            StartTime = erpc_call(
+                          Node, erlang, convert_time_unit,
+                          [SystemStartTime, native, microsecond], FromNode),
+            StartTime
+    end.
 
 -spec is_node_db_ready(Node, FromNode) -> IsReady when
       Node :: node(),
@@ -968,18 +986,26 @@ error_description({invalid_cluster_node_type, BadType}) ->
 -spec maybe_register() -> ok.
 
 maybe_register() ->
-    Backend = persistent_term:get(?PT_PEER_DISC_BACKEND, backend()),
-    case Backend:supports_registration() of
+    case should_perform_registration() of
         true ->
-            ?LOG_DEBUG(
-               "Peer discovery: registering this node",
-               #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
-            register(Backend),
-            _ = Backend:post_registration(),
-            ok;
+            Backend = persistent_term:get(?PT_PEER_DISC_BACKEND, backend()),
+            case Backend:supports_registration() of
+                true ->
+                    ?LOG_DEBUG(
+                       "Peer discovery: registering this node",
+                       #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
+                    register(Backend),
+                    _ = Backend:post_registration(),
+                    ok;
+                false ->
+                    ?LOG_DEBUG(
+                       "Peer discovery: registration is not supported, skipping it",
+                       #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
+                    ok
+            end;
         false ->
             ?LOG_DEBUG(
-               "Peer discovery: registration unsupported, skipping register",
+               "Peer discovery: registration is disabled via configuration, skipping it",
                #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
             ok
     end.
@@ -987,16 +1013,24 @@ maybe_register() ->
 -spec maybe_unregister() -> ok.
 
 maybe_unregister() ->
-    Backend = persistent_term:get(?PT_PEER_DISC_BACKEND),
-    case Backend:supports_registration() of
+    case should_perform_registration() of
         true ->
-            ?LOG_DEBUG(
-               "Peer discovery: unregistering this node",
-               #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
-            unregister(Backend);
+            Backend = persistent_term:get(?PT_PEER_DISC_BACKEND),
+            case Backend:supports_registration() of
+                true ->
+                    ?LOG_DEBUG(
+                       "Peer discovery: unregistering this node",
+                       #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
+                    unregister(Backend);
+                false ->
+                    ?LOG_DEBUG(
+                       "Peer discovery: registration is not supported, skipping unregistration",
+                       #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
+                    ok
+            end;
         false ->
             ?LOG_DEBUG(
-               "Peer discovery: registration unsupported, skipping unregister",
+               "Peer discovery: registration is disabled via configuration, skipping unregistration",
                #{domain => ?RMQLOG_DOMAIN_PEER_DISC}),
             ok
     end.
