@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2007-2024 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
+%% Copyright (c) 2007-2025 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
 %%
 
 -module(rabbit_stream_queue).
@@ -42,7 +42,6 @@
 
 -export([list_with_minimum_quorum/0]).
 
--export([set_retention_policy/3]).
 -export([restart_stream/3,
          add_replica/3,
          delete_replica/3,
@@ -312,7 +311,8 @@ consume(Q, Spec, #stream_client{} = QState0)
               consumer_tag := ConsumerTag,
               exclusive_consume := ExclusiveConsume,
               args := Args,
-              ok_msg := OkMsg} = Spec,
+              ok_msg := OkMsg,
+              acting_user := ActingUser} = Spec,
             QName = amqqueue:get_name(Q),
             rabbit_log:debug("~s:~s Local pid resolved ~0p",
                              [?MODULE, ?FUNCTION_NAME, LocalPid]),
@@ -328,7 +328,16 @@ consume(Q, Spec, #stream_client{} = QState0)
                     AckRequired = not NoAck,
                     rabbit_core_metrics:consumer_created(
                       ChPid, ConsumerTag, ExclusiveConsume, AckRequired,
-                      QName, ConsumerPrefetchCount, false, up, Args),
+                      QName, ConsumerPrefetchCount, true, up, Args),
+                    rabbit_event:notify(consumer_created,
+                                        [{consumer_tag,   ConsumerTag},
+                                         {exclusive,      ExclusiveConsume},
+                                         {ack_required,   AckRequired},
+                                         {channel,        ChPid},
+                                         {queue,          QName},
+                                         {prefetch_count, ConsumerPrefetchCount},
+                                         {arguments,      Args},
+                                         {user_who_performed_action, ActingUser}]),
                     %% reply needs to be sent before the stream
                     %% begins sending
                     maybe_send_reply(ChPid, OkMsg),
@@ -923,8 +932,31 @@ readers(QName) ->
             {node(), 0}
     end.
 
+get_writer_pid(Q) ->
+    case amqqueue:get_pid(Q) of
+        none ->
+            %% the stream is still starting; wait up to 5 seconds
+            %% and ask the coordinator as it has the Pid sooner
+            #{name := StreamId} = amqqueue:get_type_state(Q),
+            get_writer_pid(StreamId, 50);
+        Pid ->
+            Pid
+    end.
+
+get_writer_pid(_StreamId, 0) ->
+    stream_not_found;
+get_writer_pid(StreamId, N) ->
+    case rabbit_stream_coordinator:writer_pid(StreamId) of
+        {ok, Pid} ->
+            Pid;
+        _ ->
+            timer:sleep(100),
+            get_writer_pid(StreamId, N - 1)
+    end.
+
+
 init(Q) when ?is_amqqueue(Q) ->
-    Leader = amqqueue:get_pid(Q),
+    Leader = get_writer_pid(Q),
     QName = amqqueue:get_name(Q),
     #{name := StreamId} = amqqueue:get_type_state(Q),
     %% tell us about leader changes so we can fail over
@@ -946,9 +978,11 @@ init(Q) when ?is_amqqueue(Q) ->
             E
     end.
 
-close(#stream_client{readers = Readers}) ->
-    maps:foreach(fun (_, #stream{log = Log}) ->
-                         osiris_log:close(Log)
+close(#stream_client{readers = Readers,
+                     name = QName}) ->
+    maps:foreach(fun (CTag, #stream{log = Log}) ->
+                         close_log(Log),
+                         rabbit_core_metrics:consumer_deleted(self(), CTag, QName)
                  end, Readers).
 
 update(Q, State)
@@ -964,24 +998,6 @@ update_leader_pid(Pid, #stream_client{} =  State) ->
 state_info(_) ->
     #{}.
 
-set_retention_policy(Name, VHost, Policy) ->
-    case rabbit_amqqueue:check_max_age(Policy) of
-        {error, _} = E ->
-            E;
-        MaxAge ->
-            QName = queue_resource(VHost, Name),
-            Fun = fun(Q) ->
-                          Conf = amqqueue:get_type_state(Q),
-                          amqqueue:set_type_state(Q, Conf#{max_age => MaxAge})
-                  end,
-            case rabbit_amqqueue:update(QName, Fun) of
-                not_found ->
-                    {error, not_found};
-                _ ->
-                    ok
-            end
-    end.
-
 -spec restart_stream(VHost :: binary(), Queue :: binary(),
                      #{preferred_leader_node => node()}) ->
     {ok, node()} |
