@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2007-2024 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
+%% Copyright (c) 2007-2025 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
 %%
 
 -module(rabbit_auth_backend_ldap).
@@ -385,14 +385,20 @@ search_groups(LDAP, Desc, GroupsBase, Scope, DN) ->
     end.
 
 search_nested_group(LDAP, Desc, GroupsBase, Scope, CurrentDN, TargetDN, Path) ->
-    case lists:member(CurrentDN, Path) of
+    Pred0 = fun(S) ->
+                string:equal(CurrentDN, S, true)
+            end,
+    case lists:any(Pred0, Path) of
         true  ->
             ?L("recursive cycle on DN ~ts while searching for group ~ts",
                [CurrentDN, TargetDN]),
             false;
         false ->
             GroupDNs = search_groups(LDAP, Desc, GroupsBase, Scope, CurrentDN),
-            case lists:member(TargetDN, GroupDNs) of
+            Pred1 = fun(S) ->
+                        string:equal(TargetDN, S, true)
+                    end,
+            case lists:any(Pred1, GroupDNs) of
                 true  ->
                     true;
                 false ->
