@@ -2,7 +2,7 @@
 ## License, v. 2.0. If a copy of the MPL was not distributed with this
 ## file, You can obtain one at https://mozilla.org/MPL/2.0/.
 ##
-## Copyright (c) 2007-2023 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries.  All rights reserved.
+## Copyright (c) 2007-2025 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Streams.Commands.SetStreamRetentionPolicyCommand do
   alias RabbitMQ.CLI.Core.DocGuide
@@ -14,29 +14,18 @@ defmodule RabbitMQ.CLI.Streams.Commands.SetStreamRetentionPolicyCommand do
   use RabbitMQ.CLI.Core.AcceptsTwoPositionalArguments
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
-  def run([name, retention_policy], %{node: node_name, vhost: vhost}) do
-    :rabbit_misc.rpc_call(node_name, :rabbit_stream_queue, :set_retention_policy, [
-      name,
-      vhost,
-      retention_policy
-    ])
+  def run(_, _) do
+    :ok
   end
 
   use RabbitMQ.CLI.DefaultOutput
 
-  def banner([name, retention_policy], _) do
-    "Setting retention policy of stream queue #{name} to #{retention_policy} ..."
+  def banner(_, _) do
+    "DEPRECATED. This command is a no-op. Use a policy to set data retention."
   end
 
   def usage, do: "set_stream_retention_policy [--vhost <vhost>] <name> <policy>"
 
-  def usage_additional() do
-    [
-      ["<name>", "stream queue name"],
-      ["<policy>", "retention policy"]
-    ]
-  end
-
   def usage_doc_guides() do
     [
       DocGuide.streams()
@@ -45,5 +34,5 @@ defmodule RabbitMQ.CLI.Streams.Commands.SetStreamRetentionPolicyCommand do
 
   def help_section(), do: :policies
 
-  def description(), do: "Sets the retention policy of a stream queue"
+  def description(), do: "DEPRECATED. This command is a no-op. Use a policy to set data retention."
 end
