@@ -1,3 +1,4 @@
+import {oidc} from './oidc-client-ts.3.0.1.min.js';
 
 var mgr;
 var _management_logger;
@@ -85,9 +86,61 @@ function auth_settings_apply_defaults(authSettings) {
   return authSettings;
 }
 
+var oauth_settings = { oauth_enabled : false}
 
+export function set_oauth_settings(settings) {
+  oauth_settings = settings
+}
+function get_oauth_settings() {
+  return oauth_settings
+}
+
+export function oauth_initialize_if_required(state = "index") {
+  let oauth = oauth_initialize(get_oauth_settings())
+  if (!oauth.enabled) return oauth;
+  switch (state) { 
+    case 'login-callback': 
+      oauth_completeLogin(); break; 
+    case 'logout-callback': 
+      oauth_completeLogout(); break; 
+    default: 
+      oauth = oauth_initiate(oauth);
+  }
+  return oauth; 
+}
+
+export function oauth_initiate(oauth) {
+  if (oauth.enabled) {
+    if (!oauth.sp_initiated) {
+        oauth.logged_in = has_auth_credentials();        
+    } else {
+      oauth_is_logged_in().then( status => {
+        if (status.loggedIn && !has_auth_credentials()) {
+          oauth.logged_in = false;
+          oauth_initiateLogout();
+        } else {
+          if (!status.loggedIn) {
+            clear_auth();
+          } else {
+            oauth.logged_in = true;            
+            oauth.expiryDate = new Date(status.user.expires_at * 1000);  // it is epoch in seconds
+            let current = new Date();
+            _management_logger.debug('token expires in ', (oauth.expiryDate-current)/1000,
+              'secs at : ', oauth.expiryDate );
+            oauth.user_name = status.user.profile['user_name'];
+            if (!oauth.user_name || oauth.user_name == '') {
+              oauth.user_name = status.user.profile['sub'];
+            }
+            oauth.scopes = status.user.scope;
+          }
+        }
+      });
+    }
+  }
+  return oauth;
+}
 function oauth_initialize_user_manager(resource_server) {
-    oidcSettings = {
+    let oidcSettings = {
         userStore: new oidc.WebStorageStateStore({ store: window.localStorage }),
         authority: resource_server.oauth_provider_url,
         client_id: resource_server.oauth_client_id,
@@ -103,6 +156,11 @@ function oauth_initialize_user_manager(resource_server) {
           audience: resource_server.id, // required by oauth0
         },
     };
+    if (resource_server.end_session_endpoint != "") {
+      oidcSettings.metadataSeed = {
+        end_session_endpoint: resource_server.end_session_endpoint
+      }
+    }
     if (resource_server.oauth_client_secret != "") {
       oidcSettings.client_secret = resource_server.oauth_client_secret;
     }
@@ -114,7 +172,7 @@ function oauth_initialize_user_manager(resource_server) {
     oidc.Log.setLogger(console);
 
     mgr = new oidc.UserManager(oidcSettings);
-    oauth.readiness_url = mgr.settings.metadataUrl;
+//    oauth.readiness_url = mgr.settings.metadataUrl;
 
     _management_logger = new oidc.Logger("Management");
 
@@ -128,15 +186,13 @@ function oauth_initialize_user_manager(resource_server) {
       _management_logger.error("token expiring failed due to ", err);
     });
     mgr.events.addUserLoaded(function(user) {
-      console.log("addUserLoaded  setting oauth.access_token ")
-      oauth.access_token = user.access_token  // DEPRECATED
-      set_token_auth(oauth.access_token)
+      set_token_auth(user.access_token)
     });
 
 }
-function oauth_initialize(authSettings) {
+export function oauth_initialize(authSettings) {
     authSettings = auth_settings_apply_defaults(authSettings);
-    oauth = {
+    let oauth = {
       "logged_in": false,
       "enabled" : authSettings.oauth_enabled,
       "resource_servers" : authSettings.resource_servers,
@@ -144,12 +200,12 @@ function oauth_initialize(authSettings) {
     }
     if (!oauth.enabled) return oauth;
 
-    resource_server = null
+    let resource_server = null;
 
     if (oauth.resource_servers.length == 1) {
       resource_server = oauth.resource_servers[0]
     } else if (has_auth_resource()) {
-      resource_server = lookup_resource_server(get_auth_resource())
+      resource_server = lookup_resource_server(get_auth_resource(), oauth.resource_servers)
     }
 
     if (resource_server) {
@@ -184,18 +240,18 @@ function oauth_is_logged_in() {
         return { "user": user, "loggedIn": !user.expired };
     });
 }
-function lookup_resource_server(resource_server_id) {
+function lookup_resource_server(resource_server_id, resource_servers) {
   let i = 0;
 
-  while (i < oauth.resource_servers.length && oauth.resource_servers[i].id != resource_server_id) {
+  while (i < resource_servers.length && resource_servers[i].id != resource_server_id) {
     i++;
   }
-  if (i < oauth.resource_servers.length) return oauth.resource_servers[i]
+  if (i < resource_servers.length) return resource_servers[i]
   else return null
 }
 
-function oauth_initiateLogin(resource_server_id) {
-  resource_server = lookup_resource_server(resource_server_id)
+export function oauth_initiateLogin(resource_server_id) {
+  let resource_server = lookup_resource_server(resource_server_id, oauth.resource_servers)
   if (!resource_server) return;
   set_auth_resource(resource_server_id)
 
@@ -215,16 +271,15 @@ function oauth_initiateLogin(resource_server_id) {
   }
 }
 
-function oauth_redirectToHome(oauth) {
-  set_token_auth(oauth.access_token)
-
-  path = get_pref("oauth-return-to");
+function oauth_redirectToHome() {
+  let path = get_pref("oauth-return-to")
   clear_pref("oauth-return-to")
-  go_to(path)
+  go_to( !path ? "" : path)
 }
 function go_to(path) {
   location.href = rabbit_path_prefix() + "/" + path
 }
+
 function go_to_authority() {
   location.href = oauth.authority
 }
@@ -234,21 +289,44 @@ function oauth_redirectToLogin(error) {
     location.href = rabbit_path_prefix() + "/?error=" + error
   }
 }
-function oauth_completeLogin() {
-    mgr.signinRedirectCallback().then(user => oauth_redirectToHome(user)).catch(function(err) {
-        _management_logger.error(err)
-        oauth_redirectToLogin(err)
+export function oauth_completeLogin() {
+    mgr.signinRedirectCallback().then(function(user) {
+      set_token_auth(user.access_token);
+      oauth_redirectToHome();
+    }).catch(function(err) {
+      _management_logger.error(err)
+      oauth_redirectToLogin(err)
     });
 }
 
-function oauth_initiateLogout() {
+export function oauth_initiateLogout() {
   if (oauth.sp_initiated) {
-    mgr.signoutRedirect()
+    return mgr.getUser().then(user => {
+      if (user != null) {
+        mgr.metadataService.getEndSessionEndpoint().then(endpoint => {
+          if (endpoint == undefined) {
+            // Logout only from management UI
+            mgr.removeUser().then(res => {
+              clear_auth()
+              oauth_redirectToLogin()
+            })
+          }else {
+            // OpenId Connect RP-Initiated Logout
+            mgr.signoutRedirect()
+          }
+        })
+      }else {
+        clear_auth()
+        go_to_home()
+      }
+    })
+
   } else {
     go_to_authority()
   }
 }
-function oauth_completeLogout() {
+
+export function oauth_completeLogout() {
     clear_auth()
     mgr.signoutRedirectCallback().then(_ => oauth_redirectToLogin())
 }
@@ -265,8 +343,74 @@ function validate_openid_configuration(payload) {
   if (typeof payload.jwks_uri != 'string') {
     throw new Error("Missing jwks_uri")
   }
-  if (typeof payload.end_session_endpoint != 'string') {
-    throw new Error("Missing end_session_endpoint")
-  }
 
 }
+
+function warningMessageOAuthResource(oauthResource, reason) {
+  return "OAuth resource [<b>" + (oauthResource["label"] != null ? oauthResource.label : oauthResource.id) +
+    "</b>] not available. OpenId Discovery endpoint " + readiness_url(oauthResource) + reason
+}
+function warningMessageOAuthResources(commonProviderURL, oauthResources, reason) {
+  return "OAuth resources [ <b>" + oauthResources.map(resource => resource["label"] != null ? resource.label : resource.id).join("</b>,<b>")
+    + "</b>] not available. OpenId Discovery endpoint " + commonProviderURL + reason
+}
+
+export function hasAnyResourceServerReady(oauth, onReadyCallback) {
+  // Find out how many distinct oauthServers are configured
+  let oauthServers = removeDuplicates(oauth.resource_servers.filter((resource) => resource.sp_initiated))
+  oauthServers.forEach(function(entry) { console.log(readiness_url(entry)) })
+  if (oauthServers.length > 0) {   // some resources are sp_initiated but there could be idp_initiated too
+    Promise.allSettled(oauthServers.map(oauthServer => fetch(readiness_url(oauthServer)).then(res => res.json())))
+      .then(results => {
+        results.forEach(function(entry) { console.log(entry) })
+        let notReadyServers = []
+        let notCompliantServers = []
+
+        for (let i = 0; i < results.length; i++) {
+          switch (results[i].status) {
+            case "fulfilled":
+              try {
+                validate_openid_configuration(results[i].value)
+              }catch(e) {
+                console.log("Unable to connect to " + oauthServers[i].oauth_provider_url + ". " + e)
+                notCompliantServers.push(oauthServers[i].oauth_provider_url)
+              }
+              break
+            case "rejected":
+              notReadyServers.push(oauthServers[i].oauth_provider_url)
+              break
+          }
+        }
+        const spOauthServers = oauth.resource_servers.filter((resource) => resource.sp_initiated)
+        const groupByProviderURL = spOauthServers.reduce((group, oauthServer) => {
+          const { oauth_provider_url } = oauthServer;
+          group[oauth_provider_url] = group[oauth_provider_url] ?? [];
+          group[oauth_provider_url].push(oauthServer);
+          return group;
+        }, {})
+        let warnings = []
+        for(var url in groupByProviderURL){          
+          const notReadyResources = groupByProviderURL[url].filter((oauthserver) => notReadyServers.includes(oauthserver.oauth_provider_url))
+          const notCompliantResources = groupByProviderURL[url].filter((oauthserver) => notCompliantServers.includes(oauthserver.oauth_provider_url))
+          if (notReadyResources.length == 1) {
+            warnings.push(warningMessageOAuthResource(notReadyResources[0], " not reachable"))
+          }else if (notReadyResources.length > 1) {
+            warnings.push(warningMessageOAuthResources(url, notReadyResources, " not reachable"))
+          }
+          if (notCompliantResources.length == 1) {
+            warnings.push(warningMessageOAuthResource(notCompliantResources[0], " not compliant"))
+          }else if (notCompliantResources.length > 1) {
+            warnings.push(warningMessageOAuthResources(url, notCompliantResources, " not compliant"))
+          }
+        }
+        oauth.declared_resource_servers_count = oauth.resource_servers.length
+        oauth.resource_servers = oauth.resource_servers.filter((resource) =>
+          !notReadyServers.includes(resource.oauth_provider_url) && !notCompliantServers.includes(resource.oauth_provider_url))
+
+          onReadyCallback(oauth, warnings)
+
+      })
+  }else {
+    onReadyCallback(oauth, [])
+  }
+}
