@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2016-2023 VMware, Inc. or its affiliates. All rights reserved.
+%% Copyright (c) 2007-2024 Broadcom. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved. All rights reserved.
 
 -module(rabbit_mgmt_db_cache).
 
@@ -65,9 +65,14 @@ fetch(Key, FetchFun, FunArgs, Timeout) ->
     ProcName = process_name(Key),
     Pid = case whereis(ProcName) of
             undefined ->
-                {ok, P} = supervisor:start_child(rabbit_mgmt_db_cache_sup,
-                                                 ?CHILD(Key)),
-                P;
+                case supervisor:start_child(rabbit_mgmt_db_cache_sup,
+                                            ?CHILD(Key)) of
+                    {ok, P} ->
+                        P;
+                    {error, {already_started, P}} ->
+                        %% A parallel request started the cache meanwhile
+                        P
+                end;
             P -> P
           end,
     gen_server:call(Pid, {fetch, FetchFun, FunArgs}, Timeout).
