@@ -2,7 +2,7 @@
 %% License, v. 2.0. If a copy of the MPL was not distributed with this
 %% file, You can obtain one at https://mozilla.org/MPL/2.0/.
 %%
-%% Copyright (c) 2007-2024 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
+%% Copyright (c) 2007-2025 Broadcom. All Rights Reserved. The term “Broadcom” refers to Broadcom Inc. and/or its subsidiaries. All rights reserved.
 %%
 
 -module(rabbit_mgmt_format).
@@ -579,12 +579,20 @@ format_channel_details(Any) -> Any.
 
 -spec format_consumer_arguments(proplists:proplist()) -> proplists:proplist().
 format_consumer_arguments(Obj) ->
-    case pget(arguments, Obj) of
-         undefined -> Obj;
-         #{}       -> Obj;
-         []        -> pset(arguments, #{}, Obj);
-         Args      -> pset(arguments, amqp_table(Args), Obj)
-     end.
+    %% Make sure arguments is a map and not an empty list
+    Obj1 = case pget(arguments, Obj) of
+        undefined -> Obj;
+        #{}       -> Obj;
+        []        -> pset(arguments, #{}, Obj);
+        Args      -> pset(arguments, amqp_table(Args), Obj)
+    end,
+    %% Make sure channel_details is a map and not an empty list
+    case pget(channel_details, Obj1) of
+        undefined -> Obj1;
+        #{}       -> Obj1;
+        []        -> pset(channel_details, #{}, Obj1);
+        _         -> Obj1
+    end.
 
 
 parse_bool(<<"true">>)  -> true;
